/******************************************************************************
* Copyright (C) 2018 Kitsune Ral <kitsune-ral@users.sf.net>
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "converters.h"

#include <QtCore/QVariant>

using namespace QMatrixClient;

QJsonValue JsonConverter<QVariant>::dump(const QVariant& v)
{
    return QJsonValue::fromVariant(v);
}

QVariant JsonConverter<QVariant>::load(const QJsonValue& jv)
{
    return jv.toVariant();
}

QJsonObject JsonConverter<variant_map_t>::dump(const variant_map_t& map)
{
    return
#if (QT_VERSION >= QT_VERSION_CHECK(5, 5, 0))
        QJsonObject::fromVariantHash
#else
        QJsonObject::fromVariantMap
#endif
        (map);
}

variant_map_t JsonConverter<QVariantHash>::load(const QJsonValue& jv)
{
    return jv.toObject().
#if (QT_VERSION >= QT_VERSION_CHECK(5, 5, 0))
        toVariantHash
#else
        toVariantMap
#endif
        ();
}
