/******************************************************************************
 * Copyright (C) 2016 Felix Rohrbach <kde@fxrh.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#pragma once

#include "csapi/content-repo.h"

#include <QtGui/QPixmap>

namespace QMatrixClient
{
    class MediaThumbnailJob: public GetContentThumbnailJob
    {
        public:
            using GetContentThumbnailJob::makeRequestUrl;
            static QUrl makeRequestUrl(QUrl baseUrl,
                                       const QUrl& mxcUri, QSize requestedSize);

            MediaThumbnailJob(const QString& serverName, const QString& mediaId,
                              QSize requestedSize);
            MediaThumbnailJob(const QUrl& mxcUri, QSize requestedSize);

            QImage thumbnail() const;
            QImage scaledThumbnail(QSize toSize) const;

        protected:
            Status parseReply(QNetworkReply* reply) override;

        private:
            QImage _thumbnail;
    };
}  // namespace QMatrixClient
